/*
*
* Copyright (c) 2005 Sun Microsystems, Inc.
* All rights reserved.
* You may not modify, use, reproduce, or distribute this software except in compliance with the terms
* of the License at: http://developer.sun.com/berkeley_license.html
* Author : Sameer Tyagi, Sun Microsystems s.t@sun.com
* "Realizing Strategies for Document-Based Web Services With JAX-WS 2.0"
*  http://java.sun.com/developer/technicalArticles/xml/jaxrpcpatterns
*/
package com.sun.docbasedwebservices.polyvalidation;

import org.w3c.dom.Node;
import org.xml.sax.SAXException;

import javax.servlet.ServletContext;
import javax.xml.XMLConstants;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;
import javax.xml.ws.Provider;
import javax.xml.ws.Service;
import javax.xml.ws.handler.MessageContext;
import java.io.IOException;

@javax.xml.ws.ServiceMode(value=Service.Mode.PAYLOAD)
@javax.xml.ws.WebServiceProvider(wsdlLocation="WEB-INF/wsdl/POService.wsdl")
public class PurchaseOrderService  implements Provider<Source>  {
	
	@javax.annotation.Resource 
	javax.xml.ws.WebServiceContext ctx;

    private static final 		String sl = XMLConstants.W3C_XML_SCHEMA_NS_URI;
	public Source invoke(Source source) {
          try {

				SchemaFactory factory = SchemaFactory.newInstance(sl);

				ServletContext sContext=(ServletContext)(ctx.getMessageContext().get(MessageContext.SERVLET_CONTEXT));
				StreamSource  in = new StreamSource(sContext.getResourceAsStream("WEB-INF/wsdl/PurchaseOrder.xsd"));
				Schema schema = factory.newSchema(in);

				Validator v = schema.newValidator();
				            //set ErrorHandle on this validator
             v.setErrorHandler(new MyErrorHandler());
				v.validate(source);


				return source;

          } catch(IOException e) {
              e.printStackTrace();
              throw new RuntimeException("IOException, possibly caused by a missing schema file on the server endpoint", e);
          }  catch(SAXException e) {
              e.printStackTrace();
              throw new RuntimeException("Parsing/Valiation Exception ::" +e.getMessage());
          }

      }





    public static void printNodeToConsole(Node n) {
        try {
            System.out.println("\n=============================XML Received from Server =========================================");
            TransformerFactory factory = TransformerFactory.newInstance();
            Transformer transformer = factory.newTransformer();
            // also print to console
            transformer.transform(new DOMSource(n), new StreamResult(System.out));
            System.out.println("\n======================================================================");
        } catch (Exception e) {
            System.out.println(e);
        }
    }



 class MyErrorHandler implements org.xml.sax.ErrorHandler{
    
    /** Creates a new instance of MyErrorHandler */
    public MyErrorHandler() {
    }
    

    public void error(org.xml.sax.SAXParseException sAXParseException) throws POProcessingProblem {
        System.out.println("ERROR: " + sAXParseException.toString());
            throw new POProcessingProblem("There was a problem processing the request at :: " +sAXParseException.getLineNumber() +" :: message is :: "+ sAXParseException.getMessage() );
    }

    public void fatalError(org.xml.sax.SAXParseException sAXParseException) throws POProcessingProblem {
        System.out.println("FATAL ERROR: " + sAXParseException.toString());
        throw new POProcessingProblem("There was a problem processing the request at :: " +sAXParseException.getLineNumber() +" :: message is :: "+ sAXParseException.getMessage() );
    }

    public void warning(org.xml.sax.SAXParseException sAXParseException) throws org.xml.sax.SAXException {
        System.out.println("WARNING: " + sAXParseException.toString());
        // do nothing for warnings
    }
}  // end handler class


    public class POProcessingProblem extends SAXException {
        private String message = "";

        public POProcessingProblem(String message) {
            super(message);
            this.message = message;
        }

        public String getMessage() {
            return message;
        }
    }



}
